/*
 * Copyright 2013 Mark Slater
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package net.sourceforge.writexml;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.sax.TransformerHandler;
import javax.xml.transform.stream.StreamResult;
import java.io.Writer;
import java.util.Arrays;

public final class JaxpXmlWriter implements XmlWriter {

    private final TransformerHandler transformerHandler;

    public JaxpXmlWriter(final Writer writer) throws XmlWriterException {
        StreamResult streamResult = new StreamResult(writer);
        try {
            this.transformerHandler = ((SAXTransformerFactory) SAXTransformerFactory.newInstance()).newTransformerHandler();
        } catch (TransformerConfigurationException e) {
            throw new XmlWriterException("Failed configuring JaxpXmlWriter", e);
        }
        this.transformerHandler.setResult(streamResult);
        writeStartDocument();
    }

    public void write(final Document document) throws XmlWriterException {
        document.writeTo(this);
    }

    public void write(final Tag child) throws XmlWriterException {
        child.writeTo(this);
    }

    private static org.xml.sax.Attributes asXmlAttributes(final Attributes attributes) {
        final AttributesImpl resultAttributes = new AttributesImpl();
        for (Attribute attribute : attributes) {
            resultAttributes.addAttribute(attribute.namespace().asString(), attribute.name(), (attribute.namespace() == Namespace.XML ? "xml:" : "") + attribute.name(), null, attribute.value());
        }
        return resultAttributes;
    }

    public void writeStartDocument() throws XmlWriterException {
        try {
            this.transformerHandler.startDocument();
        } catch (SAXException e) {
            throw new XmlWriterException("Failed writing start document to XML", e);
        }
    }

    public void writeStartDocument(final Dtd dtd) throws XmlWriterException {
        try {
            this.transformerHandler.startDTD(dtd.name, dtd.publicId, dtd.systemId);
            this.transformerHandler.endDTD();
        } catch (SAXException e) {
            throw new XmlWriterException("Failed writing start document to XML", e);
        }
        writeStartDocument();
    }

    public void write(final String text) throws XmlWriterException {
        try {
            transformerHandler.characters(text.toCharArray(), 0, text.length());
        } catch (SAXException e) {
            throw new XmlWriterException("Failed writing text [" + text + "] to XML", e);
        }
    }

    public void writeCdata(final char[] cdata) throws XmlWriterException {
        try {
            transformerHandler.startCDATA();
            transformerHandler.characters(cdata, 0, cdata.length);
        } catch (final SAXException e) {
            throw new XmlWriterException("Failed writing CDATA section [" + Arrays.toString(cdata) + "] to XML", e);
        }
    }

    public void writeStartTag(final Tag tag) throws XmlWriterException {
        try {
            transformerHandler.startElement(tag.namespace().asString(), null, tag.name().asString(), asXmlAttributes(tag.attributes()));
        } catch (SAXException e) {
            throw new XmlWriterException("Failed writing an open tag named [" + tag.name() + "] to XML", e);
        }
    }

    public void writeEndTag(final Tag tag) throws XmlWriterException {
        try {
            transformerHandler.endElement(tag.namespace().asString(), null, tag.name().asString());
        } catch (SAXException e) {
            throw new XmlWriterException("Failed writing a close tag named [" + tag.name() + "] to XML", e);
        }
    }

    public void writeEndDocument() throws XmlWriterException {
        try {
            transformerHandler.endDocument();
        } catch (SAXException e) {
            throw new XmlWriterException("Failed writing end document to XML", e);
        }
    }

}
